/***************************************************************************
                          thegame.cpp  -  description
                             -------------------
    begin                : Mit Feb 13 13:38:33 MET 2002
    copyright            : (C) 2002 by Christoph Maciejewski
    email                : chrmac@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <qaccel.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qlcdnumber.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>
//#include <qabstractlayout.h>
#include <math.h>
#include <string.h>
//#include "warshipswindow.h" // Win32
#include "thegame.h"
#include "shootMenue.h"
#include "settings.h"
#include "filenew.xpm"

// Unter Win32 heisst die Klasse: WarshipsWindow

TheGameApp::TheGameApp(QWidget* parent, const char* name, WFlags f): QMainWindow( parent, name, f )
{
	// Window size
	xres = 800;
  yres = 550;

  // timer setting
  speed = 0;

  // Screen points
  xshot = 0;
  yshot = 90;

  // Game defaults
  life1 = life2 = 200;
  ange1 = ange2 = 45;
  ange1d = ange2d = 45 * 3.1416 / 180;
  proj1 = proj2 = 1; 	// light projectile
  speed1 = speed2 = 2;	
  prjN1[0] = 3; prjN1[1] = 6; prjN1[2] = 10;
  prjN2[0] = 3; prjN2[1] = 6; prjN2[2] = 10;
  who = 0;
  gravi = 10;
  wind = 1.0;

  //Network
  network = false;	// it is a network game? Default: false
 	typeN = 0;	// 1: Server, 0: Client. Default: 1
  strcpy(serverIP, "127.0.0.1");	// Server IP-adress, like "127.0.0.1" (default)
  strcpy(clientIP, "127.0.0.1");	// Client IP-adress, like "127.0.0.1" (default)
  dataOK = false;
  create_socket =  new_socket = -1;

  //QTime t = QTime::currentTime();
  //randomN = t.second();
	//srand(randomN);				// generate a random number sequence

 	for(int cnt=0; cnt<=60; cnt++)
	{
    windArray[cnt] = ((cnt * 20000 - cnt) % 12) - 6;
	}
	
  setCaption(tr("Warships " VERSION));
  setBackgroundColor ( QColor(0, 0, 0));

  // Install path
  picPath = "/usr/local/warships/";

  // Win32
  //picPath = ".\\Pix\\";

  resize(xres, yres);

  view = new GameArea(this, "Game", picPath);      // Die Spielflche	
  view->hide();

  frame1 = new QFrame(this);
  frame1->setBackgroundColor ( QColor(100, 100, 100));
  frame1->hide();

  frame2 = new QFrame(this);
  frame2->setBackgroundColor ( QColor(100, 100, 100));
  frame2->hide();

  //start = new QPixmap(picPath+"startPix-win.png");  // Win32
  start = new QPixmap(picPath+"startPix.jpg");   // nur provisorisch!
  startL = new QLabel(this);
 	startL->setBackgroundPixmap(*start);
 	
 	skala1 = new QPixmap(picPath+"skala.jpg");   // nur provisorisch!
  skala1L = new QLabel(this);
  skala1L->setBackgroundPixmap(*skala1);
	skala1L->hide();

  skala2L = new QLabel(this);
  skala2L->setBackgroundPixmap(*skala1);
	skala2L->hide();
		 	
 	player1 = new QPushButton ("Player1", this); 	
 	player1->hide();
 	connect( player1, SIGNAL( clicked() ), this, SLOT( newShoot1() ) );
 	
 	player2 = new QPushButton ("Player2", this); 	
 	player2->hide();
 	connect( player2, SIGNAL( clicked() ), this, SLOT( newShoot2() ) );

  gameRuns = false;    							// kein Spiel am Laufen

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initActions();
  initMenuBar();
  initStatusBar();

  autoShootTimer = new QTimer( this, "movement handler" );
  connect( autoShootTimer, SIGNAL(timeout()), this, SLOT(moveShot()) );

  autoRefreshTimer = new QTimer( this, "refresh handler" );
  connect( autoRefreshTimer, SIGNAL(timeout()), this, SLOT(refreshAfterShot()) );
}

TheGameApp::~TheGameApp()
{
}

/** initializes all QActions of the application */
void TheGameApp::initActions()
{
  QPixmap newIcon;
  newIcon = QPixmap(filenew);

  fileNew = new QAction(tr("Start new game"), newIcon, tr("&New game"), QAccel::stringToKey(tr("Ctrl+N")), this);
  fileNew->setStatusTip(tr("Start a new game"));
  fileNew->setWhatsThis(tr("New Game\n\nStart a new game"));
  connect(fileNew, SIGNAL(activated()), this, SLOT(slotNewGame()));

  fileQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
  fileQuit->setStatusTip(tr("Quits the application"));
  fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
  connect(fileQuit, SIGNAL(activated()), qApp, SLOT(quit()));

  gameSet = new QAction(tr("gameSet"), tr("&Settings"), QAccel::stringToKey(tr("Ctrl+S")), this);
  gameSet->setStatusTip(tr("Settings"));
  gameSet->setWhatsThis(tr("Settings"));
  connect(gameSet, SIGNAL(activated()), this, SLOT(slotSettings()));

  helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
  connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));

}

void TheGameApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu=new QPopupMenu();
  fileNew->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileQuit->addTo(fileMenu);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry editMenu
  editMenu=new QPopupMenu();
  gameSet->addTo(editMenu);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu=new QPopupMenu();
  helpAboutApp->addTo(helpMenu);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  menuBar()->insertItem(tr("&Game"), fileMenu);
  menuBar()->insertItem(tr("&Settings"), editMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(tr("&Help"), helpMenu);

}

void TheGameApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->message(tr("Ready."), 2000);
}


/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

//------------------------- About-Box -----------------------------
void TheGameApp::slotHelpAbout()
{
  QMessageBox::about(this,tr("About..."),
                      tr("Warships\nVersion " VERSION "\nConcept and (c) 2002 by Christoph Maciejewski
                      \nThis is a free GPL Software") );
}

//------------------------- Spiel-Ende -----------------------------
void TheGameApp::endOfGame(int win)
{
	QString winner;
	gameRuns = false;
	if(win == 1) winner = "Player 1 is the winner!";
	if(win == 2) winner = "Player 2 is the winner!";
	
  QMessageBox::about(this,tr("Game over"), winner );
  slotNewGame();
}

//-------------------------- Schiessen ---------------------------
void TheGameApp::shoot()
{
	// Buttons speren!!!
	player1->setEnabled(false);
	player2->setEnabled(false);
	
  if ( autoShootTimer->isActive() )   // schnelles Schuss-vollenden
  {
		if( network == false )
		{
			autoShootTimer->stop();
			autoShootTimer->start(0);
		}
  	return;
  }

  if(who == 0)
  {
		xshot = 0;
		autoShootTimer->start( speed + ange1/5 );   // Darstellungsgeschw. proportional zum Winkel!
  }
  else
  {
		xshot = 0;
		autoShootTimer->start( speed + ange2/5 );
  }
  qDebug("xshot: %d",xshot);
}

//--------------------- Projektil bewegen, Treffer erkennen -------------
void TheGameApp::moveShot()
{
	///////////////
  if(who == 0)
  {
  	xshot+=1;
  	yshot = (view->height()-110) - (int)(30 * ((xshot/30.0 * tan(ange1d) -
  					((gravi-wind) * xshot/30.0 * xshot/30.0) / (30.0 * speed1*5 * cos(ange1d) * cos(ange1d)))));
  	  	
  	view->drawShot(xshot+130, yshot, 0, proj1);   	
  }
  else
  {
  	xshot+=1;
  	yshot = (view->height()-110) - (int)(30 * ((xshot/30.0 * tan(ange2d) -
  					((gravi+wind) * xshot/30.0 * xshot/30.0) / (30.0 * speed2*5 * cos(ange2d) * cos(ange2d)))));
  					
  	view->drawShot(width()-130-xshot, yshot, 0, proj2);
  }
  ///////////////



  if(yshot > view->height()-106 || xshot > this->width() || xshot < 0)
  {
  	// Wasseroberflche, oder Bildschirmrand erreicht
  	autoShootTimer->stop();
  	
  	if(who == 0)
  	{
  		if(xshot+130 > this->width()-150 && xshot+130 < this->width()-50)
  		{
  			life2 -= (proj1 * proj1 * 5 + ange1/3);  	// Zerstrung von Lebensenergie abziehen
  			view->drawShot(xshot+130, view->height()-111, 1);
  			QApplication::beep(); // Treffer->beep!
  			
  		}else view->drawShot(xshot+130, view->height()-111, 2);
  		prjN1[3-proj1]--;
  	}
  	if(who == 1)
  	{
  		if(xshot+130 > this->width()-150 && xshot+130 < this->width()-50)
  		{
  			life1 -= (proj2 * proj2 * 5 + ange2/3);   // Zerstrung von Lebensenergie abziehen
  			view->drawShot(width()-130-xshot, view->height()-111, 1);
  			QApplication::beep(); // Treffer->beep!
  			
  		}else view->drawShot(width()-130-xshot, view->height()-111, 2);
  		prjN2[3-proj2]--;  	
  	}
  	
  	qDebug("who: %d xshot: %d width(): %d",who,xshot,width());   	
  	  	  	  	
  	who++;  	
  	
  	if(who > 1)
  	{
  		who = 0;
			if(network == true)
			{
				if(typeN == 1) player1->setEnabled(true);
				if(typeN == 0) player2->setEnabled(true);
			}else
			{
				player1->setEnabled(true);
				player2->setEnabled(true);
			}
			
			generateWind();		// neue Windstrke generieren
			dataOK = false;		// frs Netzwerk
		}		
		
		autoRefreshTimer->start(300);  // Flugkurve kurz sichtbar, dann refresh
		
		//----- Spielende erreicht? ----------
		if(life1 < 0) endOfGame(2);
		if(life2 < 0) endOfGame(1);
		//------------------------------------
  }
}

//----------------------- Bildschirmflche reinigen ----------------
void TheGameApp::refreshAfterShot()
{
	autoRefreshTimer->stop();	
	view->repaint();
	resizeEvent(NULL);
  view->boomL->hide();
	view->plashL->hide();
	
	if(who == 1 && gameRuns == true)
	{ 		
		startShoot();		// Player2 schiesst
	}
}

//----------------------------- Neues Spiel ------------------------
void TheGameApp::slotNewGame()
{   
  // Game defaults
  life1 = life2 = 200;
  ange1 = ange2 = 45;
  proj1 = proj2 = 1; 	// light projectile
  speed1 = speed2 = 2;	
  prjN1[0] = 3; prjN1[1] = 6; prjN1[2] = 10;
  prjN2[0] = 3; prjN2[1] = 6; prjN2[2] = 10;
  who = 0;

  QTime t = QTime::currentTime();
  randomN = t.second();


	//srand(randomN);				// generate a random number sequence

  startL->hide();
  view->show();
	setCentralWidget(view);
	player1->show();
	player2->show();
	player1->setText("Player1");
	player2->setText("Player2");
	player1->setEnabled(true);
	player2->setEnabled(true);
	frame1->show();
  frame2->show();
	skala1L->show();
	skala2L->show();
	gameRuns = true;
		
	statusBar()->message(tr("Press a button to calibrate the cannon!"));
	
  //-------- Netzwerk -------------
  if(network == true)
  {          	
  	if(connectNetwork())
  	{
  		if(typeN == 1)
  		{
  			player2->setEnabled(false);
  			player2->setText(clientIP);
  		}else
  		{
  			player1->setEnabled(false);
  			player1->setText(serverIP);
  		}
  	}
  }else create_socket =  new_socket = -1;

  //srand(randomN);  	// Zufallszahlen vordefinieren

  // resize window
  resize(xres, this->height());
  setMaximumSize(xres, this->height());
  view->setMinimumSize(xres, view->height());

  generateWind();   // Wind generieren
  view->repaint();
  resizeEvent(NULL);	
}

//---------------------- Zufallszahl fr Wind -----------------------
void TheGameApp::generateWind()
{
	//static int arrayCount = randomN;
	
	/*
	wind = (int)(-5 + (12.0*rand()/(RAND_MAX+1.0)));
 	*/

  wind = windArray[randomN];
  randomN++;
  if( randomN > 60 ) randomN = 0;

	QString w;
	w.sprintf("%d",abs((int)wind));
	view->setWindDirection((int)wind);
	wind /= 3;  	
	//qDebug("Wind: %lf", wind);	
	view->windL->setText(w);          	
}

//-------------------------- Einstellungen --------------------------
void TheGameApp::slotSettings()
{
	Settings menue (this, "menue");
	
	switch(xres)
	{
		case 640: menue.ComboBox1->setCurrentItem(0); break;
		case 800: menue.ComboBox1->setCurrentItem(1); break;
		case 1024: menue.ComboBox1->setCurrentItem(2); break;
		default: menue.ComboBox1->setCurrentItem(3); break;
	}	
	if(network == true) menue.CheckBox1->setChecked(true); else  menue.CheckBox1->setChecked(false);
	if(typeN == 0) menue.RadioButton6->setChecked(true); else  menue.RadioButton5->setChecked(true);
	menue.LineEdit3->setText(serverIP);	
	menue.slot_network();
	
	menue.exec();                                       	
	
	if(menue.ComboBox1->currentItem() == 0) { xres = 640; yres = 480; }
	if(menue.ComboBox1->currentItem() == 1) { xres = 800; yres = 600; }
	if(menue.ComboBox1->currentItem() == 2) { xres = 1024; yres = 768; }
	if(menue.CheckBox1->isChecked()) network = true;
		else {network = false; create_socket =  new_socket = -1;}
	if(menue.RadioButton6->isChecked()) typeN = 0; else typeN = 1;
	strcpy(serverIP, (menue.LineEdit3->text()).latin1());
	
	statusBar()->message(tr("Start a new game to make the settings efficacious!"), 4000);		
}

//--------------------- Schuss-Einstellungen ------------------------
void TheGameApp::newShoot1()
{        	
 	ShootMenue setParms (this, "Player 1");
 	setParms.SpinBox1->setValue(ange1);
 	setParms.SpinBox2->setValue(speed1);
 	setParms.LCDNumber3->display(prjN1[2]); 	
 	setParms.LCDNumber2->display(prjN1[1]); 	
 	setParms.LCDNumber1->display(prjN1[0]);
 	if(proj1 == 3)setParms.RadioButton1->setChecked (true);
 	if(proj1 == 2)setParms.RadioButton2->setChecked (true);
 	if(proj1 == 1)setParms.RadioButton3->setChecked (true);
 	if(prjN1[2] == 0)setParms.RadioButton3->setEnabled(false);
 	if(prjN1[1] == 0)setParms.RadioButton2->setEnabled(false);
 	if(prjN1[0] == 0)setParms.RadioButton1->setEnabled(false);
 	setParms.exec();
 	
 	ange1 = setParms.SpinBox1->value();
  ange1d = ange1 * 3.1416 / 180;
 	speed1 = setParms.SpinBox2->value();
 	prjN1[2] = setParms.LCDNumber3->intValue();	
 	prjN1[1] = setParms.LCDNumber2->intValue();
 	prjN1[0] = setParms.LCDNumber1->intValue(); 	
 	if(setParms.RadioButton1->isChecked()) proj1 = 3;
 	if(setParms.RadioButton2->isChecked()) proj1 = 2;
 	if(setParms.RadioButton3->isChecked()) proj1 = 1;
 	statusBar()->message(tr("Press \"S\" for shot!"));
	//dataOK = false;
}

void TheGameApp::newShoot2()
{     	
 	ShootMenue setParms (this, "Player 2");
 	setParms.SpinBox1->setValue(ange2);
 	setParms.SpinBox2->setValue(speed2);
 	setParms.LCDNumber3->display(prjN2[2]); 	
 	setParms.LCDNumber2->display(prjN2[1]); 	
 	setParms.LCDNumber1->display(prjN2[0]);
 	if(proj2 == 3)setParms.RadioButton1->setChecked (true);
 	if(proj2 == 2)setParms.RadioButton2->setChecked (true);
 	if(proj2 == 1)setParms.RadioButton3->setChecked (true);
 	if(prjN2[2] == 0)setParms.RadioButton3->setEnabled(false);
 	if(prjN2[1] == 0)setParms.RadioButton2->setEnabled(false);
 	if(prjN2[0] == 0)setParms.RadioButton1->setEnabled(false);
 	setParms.exec();
 	
 	ange2 = setParms.SpinBox1->value();
 	ange2d = ange2 * 3.1416 / 180;
 	speed2 = setParms.SpinBox2->value();
 	prjN2[2] = setParms.LCDNumber3->intValue();	
 	prjN2[1] = setParms.LCDNumber2->intValue();
 	prjN2[0] = setParms.LCDNumber1->intValue(); 	
 	if(setParms.RadioButton1->isChecked()) proj2 = 3;
 	if(setParms.RadioButton2->isChecked()) proj2 = 2;
 	if(setParms.RadioButton3->isChecked()) proj2 = 1;
 	statusBar()->message(tr("Press \"S\" for shot!"));
	//dataOK = false;
}

//****************************** EVENTS *****************************
//----------------- Resize -----------------------
void TheGameApp::resizeEvent (QResizeEvent *r)
{
	frame1->setGeometry(QRect(10, 50, 20, 200));
	frame2->setGeometry(QRect(this->width()-30, 50, 20, 200));
	startL->setGeometry(QRect(this->width()/2-250, this->height()/2-187, 500, 375));
	skala1L->setGeometry(QRect(10, 50+(200-life1), 20, 200-(200-life1)));
	skala2L->setGeometry(QRect(this->width()-30, 50+(200-life2), 20, 200-(200-life2)));
	player1->setGeometry(QRect(50, this->height()-70, 100, 30));
	player2->setGeometry(QRect(this->width()-150, this->height()-70, 100, 30));	
	
	xres = this->width();
}

//----------------- Key Press --------------------
void TheGameApp::keyPressEvent( QKeyEvent *k )
{ 	
	statusBar()->message(tr("Taste!."), 2000);
	switch ( k->ascii() )
	{
  	case 's':
      qDebug("\"S\" pressed");
      startShoot();
      break;
    case '.':
      qDebug("POINT");
      break;
	}
}

void TheGameApp::startShoot()
{                          	
	//-------------- Schuss vorbereiten --------------------------	
		
	if(gameRuns == true)
  {	
  	//------------ Vorrat prfen ----------
  	if(who == 0)
  	{
  		if(prjN1[3-proj1] < 1)
  		{
  			statusBar()->message(tr("Player 1: set another projectile!"), 4000);
  			dataOK = false;
  			if(typeN == 1 || network == false) return;
  		}
  		if(prjN2[3-proj2] < 1)
  		{
  			statusBar()->message(tr("Player 2: set another projectile!"), 4000);
  			dataOK = false;
  			if(typeN == 0 || network == false) return;
  		}
  	}
  	//-------------------------------------

		//--------------- Netzwerk: Daten holen ----------------
		if(network == true && dataOK == false)	
		{
			if(typeN == 1)		// Wenn Server, dann zuerst empfangen, dann senden
  		{
  			reciveData();
  			sendData();
  		}else	   					// Wenn Client, dann umgekehrt
  		{						
  			sendData();
  			reciveData();
  		}
  	
			dataOK = true;				
		}
		//------------------------------------------------------
  		
		//-------- Schuss abfeuern--------
  	shoot();
  	//--------------------------------
  		
  	//---------------------------- Kommentar -------------------------------
  	if(who == 0) statusBar()->message(tr("Player 1 fires at Player 2"), 2000);
  		else statusBar()->message(tr("Player 2 fires at Player 1"), 2000);		
  	//----------------------------------------------------------------------  		
  }
}

//------------------------------------------------

